;             :-( Apology )-:
;     :-( A 1K intro for Forever 5 )-:
;     :-( by Gasman / Hooy-Program )-:
;            :-( March 2004 )-:

	org 32768

sine equ 49152	; we like sinewaves.

; clear screen
	ld a,7
	ld (0x5c8d),a	; attr_p = white on black
	ld (0x5c48),a	; bordcr = white on black too
	call 3435		; CLS ROM routine

; generate sinewave, using ROM routine
stack_a	equ 0x2d28
fp_to_a equ 0x2dd5

	ld a,64
	out (254),a		; a good opportunity to set the border to black
	call stack_a
	rst 0x28
	db 0xc4	;store 64 into mem 4
	db 0xa3	;push half-pi
	db 0x01	;exchange
	db 0x05	;divide (-> pi/128)
	db 0xc3 ;store pi/128 into mem 3
	db 0x02	;discard it	(nice but unnecessary?)
	db 0xa0 ;push 0
	db 0x38	;return

	ld hl,sine
sinelp
	push hl

	ld h,0x58	; transform hl to an address in attribute memory,
	srl l		; for the progress bar
	srl l
	srl l
	ld (hl),39	; print some green

	rst 0x28
	db 0xe3	;recall pi/128
	db 0x0f	;add
	db 0x31	;duplicate
	db 0x1f	;sine
	db 0xe4	;recall 64 from mem 4
	db 0xa1	;push 1
	db 0x03	;subtract 1 to give 63
	db 0x04	;multiply
	db 0xe4	;recall 64 again
	db 0x0f ;add again
	db 0x38	;return
	call fp_to_a

	pop hl
	ld (hl),a
	inc l
	jr nz,sinelp

;set up interrupts
interrupt_init
	ld hl,0xBE00
	ld de,0xBE01
	ld (hl),0xBF
	ld b,e
	ld c,e	; ld bc,257
	ldir

	ld hl,0xBFBF
	ld (hl),0xC3	; opcode for JP
	ld hl,play_int
	ld (0xBFC0),hl

	ld b,10
part1lp:
	push bc
	ld b,0
zzz	halt

; simple siney thing
sine1
	ld hl,sine
	inc l
	ld (sine1+1),hl
	ld a,(hl)	; sine byte is in range 0..127
	and 0x78	; keep top 4 bits to use as y pos
	sla a
	ld d,0x2c	; high byte of attribute memory / 2
	sla a
	rl d
	ld e,a
sine2
	ld a,36
	add a,l
	ld l,a

	ld a,(hl)
	srl a
	srl a		; keep top 5 bits to use as xpos
	add a,e
	ld h,d
	ld l,a
	ld (hl),63
	xor 0x1f	; mirror x-position
	ld l,a
	ld (hl),63
	
; fade the sineything away
	xor a
	ld hl,0x5800
fadelp
	cp (hl)		; decrement each attribute byte if not already zero
	jr z,nodec
	dec (hl)
nodec
	inc hl
	bit 1,h		; loop until hl = 0x5a00 (bottom third of attributes)
	jr z,fadelp

	djnz zzz

; char printer
	ld hl,0x5020
	call printchars
	ld hl,0x5080
	call printchars
	pop bc
	djnz part1lp

init_play
	di
	ld a,0xBE
	ld i,a
	im 2
	ld a,1		; so that we'll immediately fetch a new note
	ex af,af'
	ei

	ld bc,65533
	ld de,0xBFFF

	ld a,7		; turn off AY's wave generator.
	ld b,e		; hardware synthesis is for lamers.
	out (c),a
	ld a,63
	ld b,d
	out (c),a

	exx
lp1	dec b			; the wonderful main loop.
	call z,bchan	; shadow registers b, c, d, e, h, l
	dec c			; contain period counters for each channel.
	call z,cchan	; When they reach zero,
	dec d			; do something important.
	call z,dchan
	dec e
	call z,echan
	dec h
	call z,hchan
	dec l
	call z,lchan
	jp lp1

bchan	ld b,0	; note period will be poked here -
				; reset b to the counter maximum
	exx
	ld a,8		; select AY channel A volume register
	ld b,e
	out (c),a
	ld a,8		; we will flip bit 3 to generate tone
	jr acta

cchan	ld c,0
	exx
	ld a,8		; use AY channel A here, too
	ld b,e
	out (c),a
	ld a,7		; we will flip bits 0/1/2 this time
	jr acta

acta	ld b,d
reca	xor 0	; flip the appropriate bits of the
	ld (reca+1),a	; stored volume
	out (c),a	; and write back to the AY
	exx
	ret

dchan	ld d,0	; do exactly the same thing
	exx			; for all other channels
	ld a,9
	ld b,e
	out (c),a
	ld a,8
	jr actb
echan	ld e,0
	exx
	ld a,9
	ld b,e
	out (c),a
	ld a,7
	jr actb

actb	ld b,d
recb	xor 0
	ld (recb+1),a
	out (c),a
	exx
	ret

hchan	ld h,0
	exx
	ld a,10
	ld b,e
	out (c),a
	ld a,8
	jr actc
lchan	ld l,0
	exx
	ld a,10
	ld b,e
	out (c),a
	ld a,7
	jr actc

actc	ld b,d
recc	
	xor 0
	ld (recc+1),a
	out (c),a
	exx
	ret

play_int
	ex af,af'
	dec a
	jr nz,nana
	push hl
datpos	ld hl,dat

	ld a,(hl)
	ld (bchan+1),a
	inc hl

	ld a,(hl)
	ld (cchan+1),a
	inc hl

	ld a,(hl)
	ld (dchan+1),a
	inc hl

	ld a,(hl)
	ld (echan+1),a
	inc hl

	ld a,(hl)
	ld (hchan+1),a
	inc hl

	ld a,(hl)
	ld (lchan+1),a
	inc hl

	ld a,(hl)
	or a
	jr z,fin
	inc hl
	ld (datpos+1),hl
	pop hl
nana
	ex af,af'
	ei
	ret
fin	pop hl	; remove pushed hl
	pop hl	; remove interrupt return address

; come to a crashing halt instead
	call 3435	; final cls
die	jr die

; print a line of text to screen address hl
printchars:
	exx
textpos:
	ld hl,message
	ld b,16
textlp:
	push bc ; store char counter
	push hl	; store text pos
	exx
	push hl	; store screen position
	exx
	ld l,(hl)	; get char
	ld h,0
	add hl,hl	; encoded text is premultiplied by 2
	add hl,hl	; so we only need to * by 4
				; to get the offset into the rom font.
	ld bc,0x3d00	; address of rom font
	add hl,bc
	ld b,8
charlp:
	ld a,(hl)
	inc l
	exx
	ld b,8
magnifylp:
	srl a
	rr d
	rr e
	sra d
	rr e
	djnz magnifylp
	ld (hl),d
	inc l
	ld (hl),e
	inc h
	ld (hl),e
	dec l
	ld (hl),d
	inc h
	bit 3,h
	jr z, no_text_overflow
	ld de,0xf820
	add hl,de		; bring us back to the bottom screen third, one line down
no_text_overflow:
	exx
	djnz charlp
	exx
	pop hl	; restore screen position
	inc l
	inc l	; move it on
	exx
	pop hl	; restore text pos
	inc hl	; move it on
	pop bc	; restore char counter
	djnz textlp
	; store text pointer back for next call
	ld (textpos+1),hl
	ret

dat	incbin nimrod.bin

; ah, now this is cunning. The end marker for the music needs
; to be a zero at offset 7 from the end of the music.
; But including 7 spare bytes in nimrod.bin just for this purpose
; would be *such* a waste...
; Well, guess what byte 7 of message.txt.enc happens to be? :-)

message incbin message.txt.enc

; we need lots of spaces at the end of the message, to pad
; out the last page and provide an entirely blank page
; after that. Luckily, in our encoding scheme space maps to 0,
; and we can assume that the memory following our code is
; zeroed out. Sorted.

;	the
	end
